@@ -157,10 +157,6 @@ class ObjectBase
 
     /* Quarks */
 
- protected:
-    [[nodiscard]] static GQuark instance_strings_quark();
-
- public:
     [[nodiscard]] static GQuark custom_type_quark();
     [[nodiscard]] static GQuark custom_property_quark();
     [[nodiscard]] static GQuark disposed_quark();
@@ -384,12 +380,10 @@ class ObjectInstance : public GIWrapperInstance<ObjectBase, ObjectPrototype,
         const char* for_what) const;
     [[nodiscard]] bool check_gobject_finalized(const char* for_what) const;
     GJS_JSAPI_RETURN_CONVENTION
-    bool signal_match_arguments_from_object(JSContext* cx,
-                                            JS::HandleObject props_obj,
-                                            GSignalMatchType* mask_out,
-                                            unsigned* signal_id_out,
-                                            GQuark* detail_out,
-                                            JS::MutableHandleFunction func_out);
+    bool signal_match_arguments_from_object(
+        JSContext* cx, JS::HandleObject props_obj, GSignalMatchType* mask_out,
+        unsigned* signal_id_out, GQuark* detail_out,
+        JS::MutableHandleObject callable_out);
 
  public:
     static GObject* copy_ptr(JSContext*, GType, void* ptr) {
@@ -400,8 +394,6 @@ class ObjectInstance : public GIWrapperInstance<ObjectBase, ObjectPrototype,
     bool init_custom_class_from_gobject(JSContext* cx, JS::HandleObject wrapper,
                                         GObject* gobj);
 
-    static void associate_string(GObject* obj, char* str);
-
     /* Methods to manipulate the linked list of instances */
 
  private:
