@@ -27,6 +27,7 @@
 #include <js/ValueArray.h>
 #include <js/experimental/TypedData.h>
 #include <jsapi.h>  // for InformalValueTypeName, JS_Get...
+#include <jsfriendapi.h>  // for JS_GetObjectFunction
 
 #include "gi/arg-inl.h"
 #include "gi/arg.h"
@@ -117,10 +118,8 @@ gjs_value_from_array_and_length_values(JSContext             *context,
 
     gjs_arg_set(&array_arg, g_value_get_pointer(array_value));
 
-    return gjs_value_from_explicit_array(
-        context, value_p, array_type_info,
-        no_copy ? GI_TRANSFER_NOTHING : GI_TRANSFER_EVERYTHING, &array_arg,
-        array_length.toInt32());
+    return gjs_value_from_explicit_array(context, value_p, array_type_info,
+                                         &array_arg, array_length.toInt32());
 }
 
 // FIXME(3v1n0): Move into closure.cpp one day...
@@ -162,8 +161,7 @@ void Gjs::Closure::marshal(GValue* return_value, unsigned n_param_values,
         return;
     }
 
-    JSFunction* func = callable();
-    JSAutoRealm ar(context, JS_GetFunctionObject(func));
+    JSAutoRealm ar(context, callable());
 
     if (marshal_data) {
         /* we are used for a signal handler */
@@ -275,9 +273,12 @@ void Gjs::Closure::marshal(GValue* return_value, unsigned n_param_values,
                 exit(code);
 
             // Some other uncatchable exception, e.g. out of memory
-            JSFunction* fn = callable();
-            g_error("Function %s terminated with uncatchable exception",
-                    gjs_debug_string(JS_GetFunctionDisplayId(fn)).c_str());
+            JSFunction* fn = JS_GetObjectFunction(callable());
+            std::string descr =
+                fn ? "function " + gjs_debug_string(JS_GetFunctionDisplayId(fn))
+                   : "callable object " + gjs_debug_object(callable());
+            g_error("Call to %s terminated with uncatchable exception",
+                    descr.c_str());
         }
     }
 
@@ -631,8 +632,7 @@ gjs_value_to_g_value_internal(JSContext      *context,
                 /* special case GByteArray */
                 JS::RootedObject obj(context, &value.toObject());
                 if (JS_IsUint8Array(obj)) {
-                    g_value_take_boxed(gvalue,
-                                       gjs_byte_array_get_byte_array(obj));
+                    g_value_set_boxed(gvalue, gjs_byte_array_get_byte_array(obj));
                     return true;
                 }
             } else {
@@ -881,7 +881,7 @@ gjs_value_from_g_value_internal(JSContext             *context,
                 return false;
         }
     } else if (gtype == G_TYPE_CHAR) {
-        signed char v;
+        char v;
         v = g_value_get_schar(gvalue);
         value_p.setInt32(v);
     } else if (gtype == G_TYPE_UCHAR) {
@@ -953,9 +953,8 @@ gjs_value_from_g_value_internal(JSContext             *context,
         g_arg_info_load_type(arg_info, &type_info);
         GITypeInfo* element_info = g_type_info_get_param_type(&type_info, 0);
 
-        if (!gjs_array_from_g_value_array(
-                context, value_p, element_info,
-                g_arg_info_get_ownership_transfer(arg_info), gvalue)) {
+        if (!gjs_array_from_g_value_array(context, value_p, element_info,
+                                          gvalue)) {
             gjs_throw(context, "Failed to convert array");
             return false;
         }
@@ -1031,7 +1030,7 @@ gjs_value_from_g_value_internal(JSContext             *context,
             else
                 obj = BoxedInstance::new_for_c_struct(context, info, gboxed);
         } else if (type == GI_INFO_TYPE_UNION) {
-            obj = gjs_union_from_c_union(context, info, gboxed);
+            obj = UnionInstance::new_for_c_union(context, info, gboxed);
         } else {
             gjs_throw(context, "Unexpected introspection type %d for %s",
                       info.type(), g_type_name(gtype));
