@@ -357,6 +357,10 @@ describe('Fixed-size C array', function () {
         testReturnValue('array_fixed_int', [-1, 0, 1, 2]);
         testInParameter('array_fixed_int', [-1, 0, 1, 2]);
         testOutParameter('array_fixed', [-1, 0, 1, 2]);
+        testOutParameter('array_fixed_caller_allocated', [-1, 0, 1, 2], {
+            skip: GIMarshallingTests.array_fixed_caller_allocated_out
+                ? false : 'https://gitlab.gnome.org/GNOME/gobject-introspection/-/merge_requests/370',
+        });
         testInoutParameter('array_fixed', [-1, 0, 1, 2], [2, 1, 0, -1]);
     });
 
@@ -371,6 +375,18 @@ describe('Fixed-size C array', function () {
             jasmine.objectContaining({long_: 6, int8: 7}),
         ]);
     });
+
+    it('marshals a fixed-size struct array as caller allocated out param', function () {
+        if (!GIMarshallingTests.array_fixed_caller_allocated_struct_out)
+            pending('https://gitlab.gnome.org/GNOME/gobject-introspection/-/merge_requests/370');
+
+        expect(GIMarshallingTests.array_fixed_caller_allocated_struct_out()).toEqual([
+            jasmine.objectContaining({long_: -2, int8: -1}),
+            jasmine.objectContaining({long_: 1, int8: 2}),
+            jasmine.objectContaining({long_: 3, int8: 4}),
+            jasmine.objectContaining({long_: 5, int8: 6}),
+        ]);
+    });
 });
 
 describe('C array with length', function () {
@@ -543,9 +559,6 @@ describe('Zero-terminated C array', function () {
 
         ['none', 'container', 'full'].forEach(transfer => {
             it(`marshals as a transfer-${transfer} in and out parameter`, function () {
-                if (transfer === 'full')
-                    pending('https://gitlab.gnome.org/GNOME/gobject-introspection/-/merge_requests/399');
-
                 const returnedArray =
                     GIMarshallingTests[`array_gvariant_${transfer}_in`](variantArray);
                 expect(returnedArray.map(v => v.deepUnpack())).toEqual([27, 'Hello']);
@@ -1245,9 +1258,10 @@ let VFuncTester = GObject.registerClass(class VFuncTester extends GIMarshallingT
         return i + 4;
     }
 
-    vfunc_method_str_arg_out_ret(s) {
-        return [`Called with ${s}`, 41];
-    }
+    // https://gitlab.gnome.org/GNOME/gjs/-/issues/519
+    // vfunc_method_str_arg_out_ret(s) {
+    //     return [`Called with ${s}`, 41];
+    // }
 
     vfunc_method_with_default_implementation(i) {
         this.int = i + 2;
@@ -1413,18 +1427,17 @@ describe('Virtual function', function () {
         expect(tester.method_int8_out()).toEqual(40);
     });
 
-    xit('marshals a POD out argument', function () {
+    it('marshals a POD out argument', function () {
         expect(tester.method_int8_arg_and_out_caller(39)).toEqual(42);
-    }).pend('https://gitlab.gnome.org/GNOME/gobject-introspection/-/merge_requests/263');
+    });
 
     it('marshals a callee-allocated pointer out argument', function () {
         expect(tester.method_int8_arg_and_out_callee(38)).toEqual(42);
     });
 
-    it('marshals a string out argument and return value', function () {
+    xit('marshals a string out argument and return value', function () {
         expect(tester.method_str_arg_out_ret('a string')).toEqual(['Called with a string', 41]);
-        expect(tester.method_str_arg_out_ret('a 2nd string')).toEqual(['Called with a 2nd string', 41]);
-    });
+    }).pend('https://gitlab.gnome.org/GNOME/gjs/-/issues/519');
 
     it('can override a default implementation in JS', function () {
         tester.method_with_default_implementation(40);
@@ -1647,7 +1660,7 @@ describe('Wrong virtual functions', function () {
 
     it('marshals multiple out parameters', function () {
         GLib.test_expect_message('Gjs', GLib.LogLevelFlags.LEVEL_CRITICAL,
-            'JS ERROR: Error: Function *vfunc_vfunc_multiple_out_parameters*Array*');
+            'JS ERROR: Error: *vfunc_vfunc_multiple_out_parameters*Array*');
 
         expect(tester.vfunc_multiple_out_parameters()).toEqual([0, 0]);
 
@@ -1657,7 +1670,7 @@ describe('Wrong virtual functions', function () {
 
     it('marshals a return value and one out parameter', function () {
         GLib.test_expect_message('Gjs', GLib.LogLevelFlags.LEVEL_CRITICAL,
-            'JS ERROR: Error: Function *vfunc_return_value_and_one_out_parameter*Array*');
+            'JS ERROR: Error: *vfunc_return_value_and_one_out_parameter*Array*');
 
         expect(tester.vfunc_return_value_and_one_out_parameter()).toEqual([0, 0]);
 
@@ -1667,7 +1680,7 @@ describe('Wrong virtual functions', function () {
 
     it('marshals a return value and multiple out parameters', function () {
         GLib.test_expect_message('Gjs', GLib.LogLevelFlags.LEVEL_CRITICAL,
-            'JS ERROR: Error: Function *vfunc_return_value_and_multiple_out_parameters*Array*');
+            'JS ERROR: Error: *vfunc_return_value_and_multiple_out_parameters*Array*');
 
         expect(tester.vfunc_return_value_and_multiple_out_parameters()).toEqual([0, 0, 0]);
 
@@ -1991,8 +2004,7 @@ describe('GObject properties', function () {
     testPropertyGetSetBigInt('int64', BigIntLimits.int64.min, BigIntLimits.int64.max);
     testPropertyGetSet('uint64', 42, 64);
     testPropertyGetSetBigInt('uint64', BigIntLimits.int64.max, BigIntLimits.int64.umax);
-    testPropertyGetSet('string', 'Gjs', 'is cool!',
-        'https://gitlab.gnome.org/GNOME/gobject-introspection/-/merge_requests/268');
+    testPropertyGetSet('string', 'Gjs', 'is cool!');
 
     it('get and sets out-of-range values throws', function () {
         expect(() => {
@@ -2082,7 +2094,7 @@ xdescribe('GObject signals', function () {
             const signalId = obj.connect(signalName, signalCallback);
             obj[funcName]();
             obj.disconnect(signalId);
-        }).pend('https://gitlab.gnome.org/GNOME/gobject-introspection/-/merge_requests/259');
+        });
     }
 
     testSignalEmission('boxed-gptrarray-utf8', ['0', '1', '2']);
