@@ -233,7 +233,7 @@ function _addDBusConvenience() {
 function _makeProxyWrapper(interfaceXml) {
     var info = _newInterfaceInfo(interfaceXml);
     var iname = info.name;
-    return function (bus, name, object, asyncCallback, cancellable,
+    function wrapper(bus, name, object, asyncCallback, cancellable,
         flags = Gio.DBusProxyFlags.NONE) {
         var obj = new Gio.DBusProxy({
             g_connection: bus,
@@ -253,7 +253,23 @@ function _makeProxyWrapper(interfaceXml) {
             obj.init(cancellable);
         }
         return obj;
+    }
+    wrapper.newAsync = function newAsync(bus, name, object, cancellable,
+        flags = Gio.DBusProxyFlags.NONE) {
+        const obj = new Gio.DBusProxy({
+            g_connection: bus,
+            g_interface_name: info.name,
+            g_interface_info: info,
+            g_name: name,
+            g_flags: flags,
+            g_object_path: object,
+        });
+
+        return new Promise((resolve, reject) =>
+            obj.init_async(GLib.PRIORITY_DEFAULT, cancellable ?? null).then(
+                () => resolve(obj)).catch(reject));
     };
+    return wrapper;
 }
 
 
@@ -367,7 +383,9 @@ function _handleMethodCall(info, impl, methodName, parameters, invocation) {
 function _handlePropertyGet(info, impl, propertyName) {
     let propInfo = info.lookup_property(propertyName);
     let jsval = this[propertyName];
-    if (jsval !== undefined)
+    if (jsval?.get_type_string?.() === propInfo.signature)
+        return jsval;
+    else if (jsval !== undefined)
         return new GLib.Variant(propInfo.signature, jsval);
     else
         return null;
@@ -462,6 +480,8 @@ function _warnNotIntrospectable(funcName, replacement) {
 function _init() {
     Gio = this;
 
+    Gio.Application.prototype.runAsync = GLib.MainLoop.prototype.runAsync;
+
     Gio.DBus = {
         // Namespace some functions
         get: Gio.bus_get,
@@ -596,6 +616,40 @@ function _init() {
         }
     };
 
+    Gio.InputStream.prototype.createSyncIterator = function* createSyncIterator(count) {
+        while (true) {
+            const bytes = this.read_bytes(count, null);
+            if (bytes.get_size() === 0)
+                return;
+            yield bytes;
+        }
+    };
+
+    Gio.InputStream.prototype.createAsyncIterator = async function* createAsyncIterator(count, ioPriority = Gio.PRIORITY_DEFAULT) {
+        const self = this;
+
+        function next() {
+            return new Promise((resolve, reject) => {
+                self.read_bytes_async(count, ioPriority, null, (_self, res) => {
+                    try {
+                        const bytes = self.read_bytes_finish(res);
+                        resolve(bytes);
+                    } catch (err) {
+                        reject(err);
+                    }
+                });
+            });
+        }
+
+        while (true) {
+            // eslint-disable-next-line no-await-in-loop
+            const bytes = await next(count);
+            if (bytes.get_size() === 0)
+                return;
+            yield bytes;
+        }
+    };
+
     Gio.FileEnumerator.prototype[Symbol.iterator] = function* FileEnumeratorIterator() {
         while (true) {
             try {
@@ -611,7 +665,7 @@ function _init() {
         this.close(null);
     };
 
-    Gio.FileEnumerator.prototype[Symbol.asyncIterator] = async function* AsyncFileEnumatorIterator() {
+    Gio.FileEnumerator.prototype[Symbol.asyncIterator] = async function* AsyncFileEnumeratorIterator() {
         const self = this;
 
         function next() {
